"""
@File: terminal_io.py
@Author: Azusa
@Description: 

"""
import pytest
from . import __plugin_name__

prefix_str = f"[{__plugin_name__}] "

def terminal_write(
        session: pytest.Session,
        message, prefix: bool = True,
        red: bool = False, green: bool = False,
        bold: bool = False, light: bool = False,
        blink: bool = False, invert: bool = False
):
    session.config.pluginmanager.getplugin("terminalreporter").write(
        f'{prefix_str if prefix else ""}{message}\n',
        red=red, green=green,
        bold=bold, light=light,
        blink=blink, invert=invert
    )



"""
@File: plugin.py
@Author: Azusa
@Description:

"""
import pytest
from typing import List

from .analysis import ChooseFileAnalysis
from .terminal_io import terminal_write


def pytest_addoption(parser: pytest.Parser):
    group = parser.getgroup(
        'file-choose',
        'Provide the pytest with the ability to collect use cases based on rules in text files'
    )
    group.addoption(
        "--fc",
        action="store",
        default="off",
        help="Default 'off' for file choose, option: 'on' or 'off'"
    )
    group.addoption(
        "--fc-coding",
        action="store",
        default="utf-8",
        help="File encoding, default 'utf-8'"
    )
    group.addoption(
        "--fc-path",
        action="store",
        default="./choose.json",
        help="File Path, default './choose.json'"
    )


def pytest_collection_modifyitems(session: pytest.Session, config: pytest.Config, items: List["pytest.Item"]):
    if config.getoption('--fc') == 'on':
        origin_count = len(items)
        count = 0
        path = config.getoption('--fc-path')
        terminal_write(session, '\n', prefix=False)
        terminal_write(session, f'Cases list: {path}', bold=True)
        parse = ChooseFileAnalysis(path, session, encoding=config.getoption('--fc-coding')).parse()
        if parse:
            for item in items:
                cls = item.parent.name if isinstance(item.parent, pytest.Class) else ''
                if cls and cls in parse['class']:
                    continue
                if item.name in parse['function']:
                    continue
                del items[items.index(item)]
                count += 1
            terminal_write(session, f'Filter {count} cases and collect {origin_count - count} cases', bold=True)
        else:
            terminal_write(session, 'Unsupported file format, please use JSON format file', red=True, bold=True)
            terminal_write(session, 'Not filtered')



"""
@File: test_choose.py
@Author: Azusa
@Description: 

"""


class TestDemo:
    def test_demo_1(self):
        print(1)

    def test_demo_2(self):
        print(2)


def test_demo_3():
    print(3)


def test_demo_4():
    print(4)


def test_demo_5():
    print(5)


def test_demo_6():
    print(6)


def test_demo_7():
    print(7)



"""
@File: analysis.py
@Author: Azusa
@Description: 

"""
import json
import pathlib

import pytest

from .terminal_io import terminal_write


class ChooseFileAnalysis:
    _support_type = [
        '.json'
    ]
    def __init__(self, file_path: str, session: pytest.Session, encoding: str = 'utf-8'):
        self.file_path = pathlib.PurePath(file_path)
        self.session = session

        self.f_type = self.file_path.suffix
        self.f_obj = ""
        try:
            self.f_obj = open(self.file_path.as_posix(), encoding=encoding)
        except FileNotFoundError as e:
            terminal_write(session, e.strerror, red=True)

    def __json_parse(self):
        return json.loads(self.f_obj.read())

    def parse(self):
        if isinstance(self.f_obj, str):
            return False
        if self.f_type not in self._support_type:
            print('Unsupported file format, please use JSON format file')
            return False
        if self.f_type == '.json':
            return self.__json_parse()



"""
@File: setup.py
@Author: Azusa
@Description: 

"""
import setuptools

PACKAGE = "pytest-choose"

install_requires = [
    'pytest>=4.6'
]

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name=PACKAGE,
    version="0.0.1",
    keywords="pytest",
    author="Azusa",
    author_email="naotosaka@126.com",
    description="Provide the pytest with the ability to collect use cases based on rules in text files",
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="GPLv3",
    url="https://github.com/NaoOtosaka/pytest-choose",
    packages=setuptools.find_packages(),
    install_requires=[
        'pytest>=4.6',
    ],
    classifiers=[
        'Framework :: Pytest',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent'
    ],
    project_urls={
        'Source Code': 'https://github.com/NaoOtosaka/pytest-choose',
        'Bug Tracker': 'https://github.com/NaoOtosaka/pytest-choose/issues',
    },
    entry_points={
        'pytest11': [
            'pytest-choose = pytest_choose.plugin',
        ]
    }
)
